//**************************************************************
//********************** INCLUDES ******************************
//**************************************************************
#include <Adafruit_NeoPixel.h>
#include <ArduinoMqttClient.h>
#include <WiFi.h>
#include "DHT.h"
#include <ArduinoJson.h>
#include <Adafruit_NeoPixel.h>


//**************************************************************
//******************* WIFI CONFIG ******************************
//**************************************************************
const char* ssid     = "Device_Net";  //Nombre de la red
const char* password = "ImNotHuman";  //Contaseña de la red


//**************************************************************
//******************* MQTT CONFIG ******************************
//**************************************************************
const char broker[]     = "test.mosquitto.org";
const int  port         = 1883;
const char pubTopic[]   = "testPubTopic_948372";
const char subTopic[]   = "testSubTopic_948372/+";
const char *mqtt_user   = NULL;
const char *mqtt_pass   = NULL;
WiFiClient wifiClient;
MqttClient mqttClient(wifiClient);


//**************************************************************
//*********************** NEOPIXEL *****************************
//**************************************************************
int pin         =  32;
int numPixels   =   1;
uint8_t R,G,B;
int pixelFormat = NEO_GRB + NEO_KHZ800;
Adafruit_NeoPixel *pixels;

//**************************************************************
//************************ DHT22 *******************************
//**************************************************************
#define DHTTYPE DHT22
#define DHTPIN 13     // Digital pin connected to the DHT sensor
DHT dht(DHTPIN, DHTTYPE);


//**************************************************************
//************************ GLOBAL ******************************
//**************************************************************
#define DEBUG_SERIAL Serial
int t=0;


//**************************************************************
//********************* FUNCIONES ******************************
//**************************************************************
void reconnect(void);
void setup_wifi(void);


//**************************************************************
//************************* SETUP ******************************
//**************************************************************
void setup(){
  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.begin(115200);
    while(!DEBUG_SERIAL);
  #endif
  
  setup_wifi();
  reconnect();

  dht.begin();

  pixels = new Adafruit_NeoPixel(numPixels, pin, pixelFormat);
  pixels->begin();

  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.print("Setup ended");
  #endif
}//setup


//**************************************************************
//************************** LOOP ******************************
//**************************************************************
void loop() {
  mqttClient.poll();
  if(millis()-t>500){
    t=millis();
    float humi = dht.readHumidity();
    float temp = dht.readTemperature();
    if(!(isnan(humi)||isnan(temp))){
      String payload = "";
      StaticJsonDocument<300> jsonDoc;
      jsonDoc["Temperature"] = temp;
      jsonDoc["Humidity"] = humi;
      serializeJson(jsonDoc, payload);
      mqttClient.beginMessage(pubTopic);
      mqttClient.print(payload);
      mqttClient.endMessage();
    }//if
    pixels->clear();
    pixels->setPixelColor(0,pixels->Color(R,G,B));
    //pixels->show();
  }//if
}//loop

//**************************************************************
//******************** SETUP WIFI ******************************
//**************************************************************
void setup_wifi(){
  //Connect to the wifi network
  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.print("Connecting to ");
    DEBUG_SERIAL.println(ssid);
  #endif
  WiFi.begin(ssid, password);
  while (WiFi.status() != WL_CONNECTED){
    delay(500);
    #ifdef DEBUG_SERIAL
      DEBUG_SERIAL.print(".");
    #endif
  }//while
  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.println("\nWiFi connected");
    DEBUG_SERIAL.println("IP address: ");
    DEBUG_SERIAL.println(WiFi.localIP());
  #endif
  return;
}//setup_wifi


//**************************************************************
//************************** MQTT ******************************
//**************************************************************
void Callback(int messageSize){
  StaticJsonDocument<256> doc;
  deserializeJson(doc, mqttClient);
  R = doc["R"];
  G = doc["G"];
  B = doc["B"];
  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.println();
    DEBUG_SERIAL.print("Topic:");
    DEBUG_SERIAL.println(mqttClient.messageTopic());
    DEBUG_SERIAL.print("Neopixel Values:\t");
    DEBUG_SERIAL.print("R:");
    DEBUG_SERIAL.print(R);
    DEBUG_SERIAL.print(",G:");
    DEBUG_SERIAL.print(G);
    DEBUG_SERIAL.print(",B:");
    DEBUG_SERIAL.println(B);
  #endif
  return;
}//Callback

void reconnect(){
  //Connect client
  #ifdef DEBUG_SERIAL
    DEBUG_SERIAL.println("Connecting to the broker");
  #endif
  if (!mqttClient.connect(broker, port)) {
    #ifdef DEBUG_SERIAL
      DEBUG_SERIAL.print("MQTT connection failed! Error code = ");
      DEBUG_SERIAL.println(mqttClient.connectError());
    #endif
    ESP.restart();
  }
  // set the message receive callback
  mqttClient.onMessage(Callback);
  mqttClient.subscribe(subTopic);
  return;
}//reconnect
